#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _default = _interopDefault(require('lodash/has'));
var minimist = _interopDefault(require('minimist'));
var _default2 = _interopDefault(require('lodash/fromPairs'));
var _default$1 = _interopDefault(require('lodash/repeat'));
var fs = _interopDefault(require('fs-promise'));
var path = _interopDefault(require('path'));
var chokidar = _interopDefault(require('chokidar'));
var chalk = _interopDefault(require('chalk'));
var open = _interopDefault(require('open'));
var express = _interopDefault(require('express'));

function help () {
	console.log("halp plz");
	process.exit(0);
}

function version (argv, Temple) {
	console.log("templejs %s", Temple.version);
	process.exit(0);
}

function fetchFile(f) {
	return fs.readFile(path.resolve(f), {
		encoding: "utf-8"
	}).then(function (src) {
		return [f, src];
	});
}

var STDIN;

function fetchStdin() {
	if (STDIN != null) return Promise.resolve(["_input.js", STDIN]);
	return new Promise(function (resolve, reject) {
		var src = "";
		process.stdin.setEncoding("utf-8");
		process.stdin.on("data", function (d) {
			return src += d;
		});
		process.stdin.on("end", function () {
			return resolve(["_input.js", STDIN = src]);
		});
		process.stdin.on("error", reject);
	});
}

function printError(e) {
	if (e.location) {
		console.error("\n" + chalk.bold(e.name) + "\n");
		var line = e.location.start.line;
		var col = e.location.start.column;
		var file = path.relative(process.cwd(), e.filename);
		console.error("  " + chalk.underline(file + ":" + line + ":" + col + "\n"));
		var lines = e.source.split(/\r?\n/g);

		var numoffset = (line + 3).toString().length;

		for (var i = 3; i > 0; i--) {
			if (line - i <= 0) continue;
			var l = (line - i).toString();
			console.error("  " + _default$1(" ", numoffset - l.length) + l + ": " + lines[l - 1]);
		}

		console.error("  " + _default$1(" ", numoffset - line.toString().length) + line + ": " + lines[line - 1]);
		console.error("  " + _default$1(" ", numoffset + 2) + lines[line - 1].substr(0, col - 1).replace(/\S/g, " ") + chalk.red("↑"));

		for (var i = 1; i <= 3; i++) {
			if (line + i > lines.length) continue;
			var l = (line + i).toString();
			console.error("  " + _default$1(" ", numoffset - l.length) + l + ": " + lines[l - 1]);
		}

		console.error(chalk.red("\n  " + e.message + "\n"));
	} else {
		console.error(e.stack || e);
	}
}

function panic(e) {
	printError(e);
	process.exit(1);
}

function compile(argv, Temple, onBuild) {
	var timeout = undefined,
	    watcher = undefined;
	var building = false;
	var files = argv._.map(function (f) {
		return path.resolve(f);
	});
	var watchedFiles = files.slice(0);
	var hasStdin = !process.stdin.isTTY;

	function build() {
		if (building) return invalidate();
		building = true;
		var done = function done() {
			return building = false;
		};

		var p = files.map(fetchFile);
		if (hasStdin) p.push(fetchStdin());

		return Promise.all(p).then(function (r) {
			return Temple.compile(_default2(r), {
				format: argv.format,
				async: true
			});
		}).then(function (result) {
			done();
			if (watcher) {
				watcher.unwatch(watchedFiles);
				watcher.add(watchedFiles = result.includes);
			}
			if (onBuild) onBuild(result);
			return result;
		}, function (e) {
			done();
			throw e;
		});
	}

	function invalidate() {
		if (timeout) return;
		timeout = setTimeout(function () {
			timeout = null;
			build().catch(printError);
		}, 500);
	}

	if (argv.watch) {
		watcher = chokidar.watch(watchedFiles, {
			ignoreInitial: true,
			persistent: true
		});

		watcher.on("all", invalidate);
		watcher.on("error", panic);
	}

	return build();
}

function compile$1 (argv, Temple) {
	return compile(argv, Temple, function (result) {
		var mapFile = argv["source-map"];
		var output = argv.output ? path.resolve(argv.output) : null;
		var p = [];
		var code = undefined;

		if (!mapFile) {
			code = result.code;
		} else {
			if (mapFile === "inline") mapFile = null;
			code = result.toString(mapFile);

			if (typeof mapFile === "string") {
				if (output) mapFile = path.resolve(path.dirname(output), mapFile);else mapFile = path.resolve(mapFile);
				p.push(fs.writeFile(mapFile, result.map.toString()));
			}
		}

		if (output) return fs.writeFile(output, code);
		console.log(code);
	}).catch(function (e) {
		printError(e);
		if (!argv.watch) process.exit(1);
	});
}

var html = "<!DOCTYPE html>\n\n<html lang=\"en-US\">\n\t<head>\n\t\t<meta charset=\"UTF-8\" />\n\t\t<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\" />\n\t</head>\n\t<body>\n\t\t<script type=\"text/javascript\" src=\"/temple.js\"></script>\n\t\t<script type=\"text/javascript\" src=\"/template.js\"></script>\n\t</body>\n</html>";

function run (argv, Temple) {
	var source = undefined;
	var app = express();

	app.get("/", function (req, res) {
		return res.send(html);
	});
	app.use(express.static(__dirname + "/dist"));
	app.get("/template.js", function (req, res) {
		return res.type("js").send(source);
	});

	argv.format = "iife";
	argv.watch = true;

	return compile(argv, Temple, function (result) {
		source = result.toString();
	}).then(function () {
		// let server = http.createServer(onRequest());
		var server = app.listen(argv.port || 6392, "127.0.0.1", function () {
			var addr = server.address();
			var url = "http://" + addr.address + ":" + addr.port;
			if (argv.open) open(url);
			console.log("HTTP server listening at " + url + ".");
			console.log("Type Ctrl-C to stop the server.");
		});

		server.on("error", function (e) {
			printError(e);
			process.exit(1);
		});
	}).catch(printError);
}

var clientSource = "!function(){var e={},n=Temple.idom,t=Temple.decorators;!function(){var e=document.createElement(\"style\");e.innerHTML=\"\\nbody {\\n\tmargin: 0;\\n\tpadding: 100px 0 0 0;\\n\tfont-size: 16px;\\n\tline-height: 1.5;\\n}\\n\\nnav {\\n\tposition: fixed;\\n\twidth: 100%;\\n\ttop: 0;\\n\tleft: 0;\\n\theight: 100px;\\n\tbackground-color: green;\\n}\\n\\nnav .page-header {\\n\\n}\\n\\n@media(max-height: 900px) {\\n\tbody {\\n\t\tpadding-top: 70px;\\n\t}\\n\\n\tnav {\\n\t\theight: 70px;\\n\t}\\n}\\n\",document.head.appendChild(e)}(),e.header=Temple.create(\"header\",function(e,o){n.text(\" \"),n.elementOpen(\"nav\",o+\"-1\"),n.text(\" \"),n.elementOpen(\"div\",\"1\"),t.render(e,\"class\",\"page-header\"),n.text(\" \"),n.elementOpen(\"h1\",\"1\"),n.text(\"Temple Playground\"),n.elementClose(\"h1\"),n.text(\" \"),n.elementClose(\"div\"),n.text(\" \"),n.elementClose(\"nav\"),n.text(\" \")})}(),function(){function e(e,n){for(;e;){if(e.nodeType===Node.ELEMENT_NODE)if(n.nodeType===Node.ELEMENT_NODE){if(n===e)return e}else if(\"string\"==typeof n&&e.matches(n))return e;e=e.parentNode}}var n={},t=Temple.idom,o=Temple.decorators;!function(){var e=document.createElement(\"style\");e.innerHTML=\"\\n.ace-editor {\\n\twidth: 100%;\\n\theight: 100%;\\n}\\n\",document.head.appendChild(e)}(),function(){var e=document.createElement(\"script\");e.type=\"text/javascript\",e.src=\"https://cdnjs.cloudflare.com/ajax/libs/ace/1.2.3/ace.js\",document.head.appendChild(e)}(),n.editor=Temple.create(\"editor\",function(e,n){t.text(\" \"),Temple.With(e.lookup(\"buttonCtx\"),e,function(e){t.text(\" \"),Temple.render(\"button-bar\",e,n+\"-1-1\"),t.text(\" \")}),t.text(\" \"),t.elementOpen(\"div\",n+\"-3\"),o.render(e,\"class\",\"ace-editor\"),o.render(e,\"ace-editor\",function(e){return[e.lookup(\"content\"),e.lookup(\"resetEditor\")]}),t.elementClose(\"div\"),t.text(\" \")}),n.editor.helpers({buttonCtx:{contentView:\"editor-navbar\"}}),n.editor.decorate(\"ace-editor\",function(e,n,t){var o=ace.edit(e.target);o.setTheme(\"ace/theme/monokai\"),o.$blockScrolling=1/0;var r=o.getSession();r.setUseWorker(!1),r.setMode(\"ace/mode/html\"),r.setUseWrapMode(!0);var i=!1;Trackr.autorun(function(){var e=n.get();i||o.setValue(e,-1),i=!1}),o.on(\"change\",_.debounce(function(){var e=o.getValue();n.get()!==e&&(i=!0,e?n.set(e):t())},1e3)),e.comp.onInvalidate(function(){o.destroy()})},{defer:!0}),n[\"editor-navbar\"]=Temple.create(\"editor-navbar\",function(e,n){t.text(\" \"),t.elementOpen(\"div\",n+\"-1\"),o.render(e,\"class\",\"dropdown btn\"),o.render(e,\"on-click\",\"toggle-dropdown\"),t.text(\" \"),t.elementOpen(\"span\",\"1\"),o.render(e,\"class\",\"handle\"),t.elementOpen(\"i\",\"0\"),o.render(e,\"class\",\"icon icon-caret-down\"),t.elementClose(\"i\"),t.elementClose(\"span\"),t.text(\" \"),t.elementOpen(\"span\",\"3\"),o.render(e,\"class\",function(e){return[\"value\",e.lookup(\"selectedExample\").get()?\"\":Temple.utils.joinValues(\" value-hide\")].map(Temple.utils.toString).join(\"\")}),t.text(\" \"),t.autotext(function(){return e.lookup(\"selectedExample\").get()}),t.text(\" \"),t.elementClose(\"span\"),t.text(\" \"),Temple.Element(\"div\",\"5\",e,function(e){o.render(e,\"class\",\"dropdown-menu\"),t.text(\" \"),Temple.Each(e.lookup(\"examples\"),[\"ex\"],e,function(e,n){t.text(\" \"),t.elementOpen(\"a\",\"1-\"+n+\"-1\"),o.render(e,\"href\",\"#\"),o.render(e,\"on-click\",function(e){return[\"select-item\",e.lookup(\"fetchExample\"),e.lookup(\"ex\")]}),t.autotext(function(){return e.lookup(\"ex\")}),t.elementClose(\"a\"),t.text(\" \")}),t.text(\" \")}),t.text(\" \"),t.elementClose(\"div\"),t.text(\" \"),t.elementOpen(\"a\",n+\"-3\"),o.render(e,\"href\",\"#\"),o.render(e,\"class\",\"btn btn-right\"),t.elementOpen(\"i\",\"0\"),o.render(e,\"class\",\"icon icon-share\"),t.elementClose(\"i\"),t.text(\" Share\"),t.elementClose(\"a\"),t.text(\" \"),t.elementOpen(\"a\",n+\"-5\"),o.render(e,\"href\",\"#\"),o.render(e,\"class\",function(e){return[\"btn btn-right\",e.lookup(\"content\").get()?\"\":Temple.utils.joinValues(\" btn-hide\")].map(Temple.utils.toString).join(\"\")}),o.render(e,\"on-click\",function(e){return[\"reset-editor\",e.lookup(\"resetEditor\")]}),t.text(\" \"),t.elementOpen(\"i\",\"1\"),o.render(e,\"class\",\"icon icon-reset icon-fw\"),t.elementClose(\"i\"),t.text(\" Start Over \"),t.elementClose(\"a\"),t.text(\" \")}),n[\"editor-navbar\"].use(\"actions\"),n[\"editor-navbar\"].actions({\"select-item\":function(e,n,t){e.original.preventDefault(),n(t)},\"toggle-dropdown\":function(n){e(n.original.target,\".dropdown-menu a\")||n.original.preventDefault();var t=n.target;t.classList.toggle(\"open\"),setTimeout(function(){var n;document.addEventListener(\"click\",n=function(o){document.removeEventListener(\"click\",n),e(o.target,t)||t.classList.remove(\"open\")})},0)},\"reset-editor\":function(e,n){e.original.preventDefault(),n()}})}(),function(){var e={},n=Temple.idom,t=Temple.decorators;!function(){var e=document.createElement(\"style\");e.innerHTML=\"\\niframe.preview {\\n\tborder: none;\\n\twidth: 100%;\\n\theight: 100%;\\n\tdisplay: block;\\n}\\n\",document.head.appendChild(e)}();var o=new Trackr.Dependency;e.preview=Temple.create(\"preview\",function(e,o){n.text(\" \"),Temple.With(e.lookup(\"buttonCtx\"),e,function(e){n.text(\" \"),Temple.render(\"button-bar\",e,o+\"-1-1\"),n.text(\" \")}),n.text(\" \"),n.elementOpen(\"iframe\",o+\"-3\"),t.render(e,\"class\",\"preview\"),t.render(e,\"preview-content\",function(e){return e.lookup(\"content\")}),n.elementClose(\"iframe\"),n.text(\" \")}),e.preview.helpers({buttonCtx:{contentView:\"preview-navbar\"}}),e.preview.decorate(\"preview-content\",function(e,n){var t,r,i=e.target;t=i.contentWindow||i,r=i.contentDocument?i.contentDocument:t.document,r.write(\"\"),r.close(),Trackr.autorun(function(){o.depend(),r.open(),r.write([\"<!DOCTYPE html>\",'<html lang=\"en-US\">',\"\t<head>\",'\t\t<meta charset=\"UTF-8\" />','\t\t<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\" />',\"\t</head>\",\"\t<body>\",'\t\t<script type=\"text/javascript\" src=\"/temple.js\"></script>','\t\t<script type=\"text/javascript\">',Temple.compile(n.get()).toString(),\"\t\t</script>\",\"\t</body>\",\"</html>\"].join(\"\\n\")),r.close()})}),e[\"preview-navbar\"]=Temple.create(\"preview-navbar\",function(e,o){n.text(\" \"),n.elementOpen(\"a\",o+\"-1\"),t.render(e,\"href\",\"#\"),t.render(e,\"class\",\"btn btn-right\"),t.render(e,\"on-click\",\"reload-preview\"),n.elementOpen(\"i\",\"0\"),t.render(e,\"class\",\"icon icon-refresh\"),n.elementClose(\"i\"),n.elementClose(\"a\"),n.text(\" \")}),e[\"preview-navbar\"].use(\"actions\"),e[\"preview-navbar\"].actions(\"reload-preview\",function(e){e.original.preventDefault(),o.changed()})}(),function(){var e={},n=Temple.idom,t=Temple.decorators;!function(){var e=document.createElement(\"style\");e.innerHTML=\"\\n.button-bar {\\n\tposition: absolute;\\n\ttop: 0;\\n\tleft: 0;\\n\twidth: 100%;\\n\theight: 50px;\\n\tbackground-color: #ebebeb;\\n\tpadding: 0.375rem;\\n\tfont-size: 1pt;\\n\tz-index: 100;\\n}\\n\\n.btn, .btn-input {\\n\tdisplay: inline-block;\\n\theight: 100%;\\n\tline-height: calc(50px - (0.375rem * 4));\\n\tpadding: 0.375rem 0.875rem;\\n\tborder-radius: 3px;\\n\tmargin-right: 0.375rem;\\n\tfont-size: 0.8rem;\\n\tfloat: left;\\n\ttransition: opacity 100ms;\\n\topacity: 1;\\n}\\n\\n.btn {\\n\tbackground-color: #dadada;\\n\ttext-decoration: none;\\n\tcolor: #555;\\n\tcursor: pointer;\\n}\\n\\n.btn:hover {\\n\tbackground-color: #ccc;\\n}\\n\\n.btn-hide {\\n\topacity: 0;\\n}\\n\\n.btn-input {\\n\tbackground-color: #f8f8f8;\\n\tcolor: #666;\\n\tborder: none;\\n\toutline: none;\\n}\\n\\n.btn-input:focus {\\n\tbackground-color: white;\\n\tcolor: #333;\\n}\\n\\n.btn-right {\\n\tfloat: right;\\n\tmargin-left: 0.375rem;\\n\tmargin-right: 0;\\n}\\n\\n.dropdown {\\n\tposition: relative;\\n\tfont-size: 1rem;\\n\tdisplay: inline-block;\\n}\\n\\n.dropdown.btn {\\n\tpadding: 2px;\\n}\\n\\n.dropdown .handle,\\n.dropdown .value {\\n\tdisplay: inline-block;\\n\tpadding: calc(0.375rem - 2px) 0.875rem;\\n\tfloat: left;\\n}\\n\\n.dropdown .value {\\n\tbackground-color: white;\\n}\\n\\n.dropdown .value.value-hide {\\n\tdisplay: none;\\n}\\n\\n.dropdown .dropdown-menu {\\n\tdisplay: none;\\n\tposition: absolute;\\n\ttop: 100%;\\n\tleft: 0;\\n\tz-index: 9999;\\n\tmin-width: 160px;\\n\tbackground-color: white;\\n\tmargin: 2px 0 0;\\n\tpadding: 5px 0;\\n\tborder-radius: 3px;\\n\tborder: 1px solid #ddd;\\n}\\n\\n.dropdown.open .dropdown-menu {\\n\tdisplay: block;\\n}\\n\\n.dropdown .dropdown-menu a {\\n\tdisplay: block;\\n\tpadding: 3px 20px;\\n\tcolor: #333;\\n\ttext-decoration: none;\\n    white-space: nowrap;\\n}\\n\\n.dropdown .dropdown-menu a:hover {\\n\tbackground-color: rgba(0, 0, 0, 0.05);\\n}\\n\\n.dropdown .dropdown-menu .divider {\\n\tborder-bottom: 1px solid #ddd;\\n\tmargin: 3px 0;\\n}\\n\",document.head.appendChild(e)}(),e[\"button-bar\"]=Temple.create(\"button-bar\",function(e,o){n.text(\" \"),Temple.Element(\"div\",o+\"-1\",e,function(e){t.render(e,\"class\",\"button-bar\"),Temple.render(e.lookup(\"contentView\"),e,\"0\")}),n.text(\" \")})}(),function(){function e(){var e=new XMLHttpRequest;e.addEventListener(\"load\",function(){d.splice(0,d.length),Array.isArray(e.response)&&d.push.apply(d,e.response)}),e.open(\"GET\",\"/examples\"),e.responseType=\"json\",e.send()}function n(e){var n=new XMLHttpRequest;n.addEventListener(\"load\",function(){l.set(e),a.set(n.responseText)}),n.open(\"GET\",\"/examples/\"+encodeURIComponent(e)),n.send()}function t(){l.set(),a.set(\"\")}var o={},r=Temple.idom,i=Temple.decorators;!function(){var e=document.createElement(\"style\");e.innerHTML=\"\\nbody, html {\\n\theight: 100%;\\n\tfont-family: sans-serif;\\n}\\n\\n*, *:before, *:after {\\n\tbox-sizing: border-box;\\n}\\n\\n.content-box {\\n\tfloat: left;\\n\twidth: 50%;\\n\theight: 100%;\\n\tposition: relative;\\n\tpadding-top: 50px;\\n}\\n\\n.content-box:nth-of-type(1) {\\n\tborder-right: 2px solid #cccccc;\\n}\\n\",document.head.appendChild(e)}(),function(){var e=document.createElement(\"script\");e.type=\"text/javascript\",e.src=\"https://use.fonticons.com/7a9027f4.js\",document.head.appendChild(e)}(),function(){var e=document.createElement(\"script\");e.type=\"text/javascript\",e.src=\"https://cdnjs.cloudflare.com/ajax/libs/lodash.js/4.5.0/lodash.min.js\",document.head.appendChild(e)}(),o.layout=Temple.create(\"layout\",function(e,n){r.text(\" \"),Temple.render(\"header\",e,n+\"-1\"),r.text(\" \"),Temple.Element(\"div\",n+\"-3\",e,function(e){i.render(e,\"class\",\"content-box editor-box\"),Temple.render(\"editor\",e,\"0\")}),r.text(\" \"),Temple.Element(\"div\",n+\"-5\",e,function(e){i.render(e,\"class\",\"content-box preview-box\"),Temple.render(\"preview\",e,\"0\")}),r.text(\" \")});var a=new Temple.Variable(\"\"),d=new Temple.List,l=new Temple.Variable;document.addEventListener(\"DOMContentLoaded\",e),window.addEventListener(\"load\",function(){Temple.paint(\"layout\",\"body\",{content:a,examples:d,selectedExample:l,fetchExample:n,resetEditor:t})})}();";

var app = express();
var html$1 = "<!DOCTYPE html>\n\n<html lang=\"en-US\">\n\t<head>\n\t\t<meta charset=\"UTF-8\" />\n\t\t<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\" />\n\t</head>\n\t<body>\n\t\t<script type=\"text/javascript\" src=\"/temple.js\"></script>\n\t\t<script type=\"text/javascript\" src=\"/playground.js\"></script>\n\t</body>\n</html>";

app.get("/", function (req, res) {
	res.type("html").send(html$1);
});

app.get("/temple(.min)?.js", function (req, res) {
	res.sendFile(path.join(__dirname, "dist", req.path));
});

app.get("/playground.js", function (req, res) {
	res.type("js").send(clientSource);
});

var examplesDir = path.join(__dirname, "src/playground/examples");

app.get("/examples", function (req, res, next) {
	fs.readdir(examplesDir).then(function (examples) {
		return Promise.all(examples.map(function (f) {
			if (f[0] === ".") return;
			return fs.stat(path.join(examplesDir, f)).then(function (s) {
				return s.isFile() ? f : null;
			});
		}));
	}).then(function (r) {
		res.send(r.filter(Boolean));
	}).catch(next);
});

app.get("/examples/:name", function (req, res) {
	res.type("text").sendFile(path.join(examplesDir, req.params.name));
});

function playground (argv) {
	var server = app.listen(argv.port || 6392, "127.0.0.1", function () {
		var addr = server.address();
		var url = "http://" + addr.address + ":" + addr.port;
		console.log("HTTP server listening at " + url + ".");
		console.log("Type Ctrl-C to stop the server.");
	});

	server.on("error", function (e) {
		printError(e);
		process.exit(1);
	});
}

var Temple = require("./");

var commands = {
	help: help, version: version,
	compile: compile$1, run: run, playground: playground
};

// command aliases
commands.build = commands.compile;
commands.start = commands.exec = commands.run;
commands.open = function (argv) {
	argv.open = true;
	return commands.run.apply(this, arguments);
};

var argv = minimist(process.argv.slice(2), {
	string: ["output", "format"],
	boolean: ["help", "version", "open", "watch"],
	alias: {
		h: "help", H: "help",
		v: "version", V: "version",
		m: "source-map",
		o: "output",
		w: "watch",
		f: "format"
	}
});

var cmd = undefined;
if (argv.help) cmd = "help";else if (argv.version) cmd = "version";else if (argv._.length) {
	var tcmd = argv._[0];
	if (_default(commands, tcmd)) {
		cmd = tcmd;
		argv._.shift();
	} else {
		cmd = "compile";
	}
} else {
	cmd = process.stdin.isTTY ? "help" : "compile";
}

commands[cmd](argv, Temple);